@@ -1,18 +1,21 @@
 import logging
 import time
 import traceback
+import json
+
 # import threading
 
 from .logged_requests import LoggedRequests
 from pprint import pformat
 from pprint import pprint
-
+from jsoncomparison import Compare
 
 try:
     import settings
 except ImportError:
     traceback.print_exc()
 
+
 class BaseTestCase(object):
     """
 BaseTestCase is the superclass for all the backend(Non UI) and frontend(UI) Testcases and Mobile Testcases
@@ -106,13 +109,6 @@ BaseTestCase is the superclass for all the backend(Non UI) and frontend(UI) Test
         """
         pass
 
-    # def tearDown(self):
-    #     """
-    #     def teardown(self):
-    #         ...
-    #     """
-    #     pass
-
     # def get_testcase_id(self):
     #     return self.testcase_id
 
@@ -156,20 +152,27 @@ BaseTestCase is the superclass for all the backend(Non UI) and frontend(UI) Test
                         "assert_contains => source_val: %s, target_val: %s    => Passed" % (source_val, target_val))
                 else:
                     self.logger.error(
-                        "assert_contains => source_val: %s, target_val: %s    -------------------------> Failed" % (source_val, target_val))
-                    raise RuntimeError("assert_contains => source_val: %s, target_val: %s    -------------------------> Failed" % (source_val, target_val))
+                        "assert_contains => source_val: %s, target_val: %s    -------------------------> Failed" % (
+                            source_val, target_val))
+                    raise RuntimeError(
+                        "assert_contains => source_val: %s, target_val: %s    -------------------------> Failed" % (
+                            source_val, target_val))
             else:
                 self.logger.error(
                     "assert_contains => source_val: %s, target_val: %s     -------------------------> Failed, "
                     "the type of params should be on of [str, int, float] " % (source_val, target_val))
-                raise RuntimeError("assert_contains => source_val: %s, target_val: %s     -------------------------> Failed, "
+                raise RuntimeError(
+                    "assert_contains => source_val: %s, target_val: %s     -------------------------> Failed, "
                     "the type of params should be on of [str, int, float] " % (source_val, target_val))
 
         except AssertionError as e:
             e.args += ('assert_contains => source_val is %s' % source_val, 'target_val is %s' % target_val)
             self.logger.error(
-                "assert_contains => source_val: %s, target_val: %s      -------------------------> Failed" % (source_val, target_val))
-            raise RuntimeError("assert_contains => source_val: %s, target_val: %s      -------------------------> Failed" % (source_val, target_val))
+                "assert_contains => source_val: %s, target_val: %s      -------------------------> Failed" % (
+                    source_val, target_val))
+            raise RuntimeError(
+                "assert_contains => source_val: %s, target_val: %s      -------------------------> Failed" % (
+                    source_val, target_val))
 
     def assert_is_not_none(self, actual_val):
         try:
@@ -179,3 +182,45 @@ BaseTestCase is the superclass for all the backend(Non UI) and frontend(UI) Test
             e.args += ('expected value is not None', 'actual_val is %s' % actual_val)
             self.logger.info("assert_is_not_none(%s)  -- Failed" % (actual_val))
             raise
+
+    def compare_json_and_return_diff(self, expected_json, actual_json):
+        self.logger.info(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> compare_json: ")
+        self.logger.info("------ expected_json: ")
+        self.logger.info(self.pformat(expected_json))
+        self.logger.info("------ actual_json: ")
+        self.logger.info(self.pformat(actual_json))
+
+        diff = Compare().check(expected_json, actual_json)
+        self.logger.info("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< compare_json diff: ")
+        self.logger.info(self.pformat(diff))
+
+        # 判断逻辑交给用户，这里直接返回diff
+        return diff
+
+    def compare_json_and_assert_equal(self, expected_json, actual_json):
+        diff = self.compare_json_and_return_diff(expected_json, actual_json)
+        self.pprint(diff)
+        assert diff == {}
+
+    def compare_json_ignore_order(self, expected_json, actual_json):
+        # 忽略json体内的顺序, 即认为： { "name": "lintest", "version": 1.0 }  和 {"version": 1.0, "name": "lintest" } 是等价的
+
+        self.logger.info(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> compare_json_ignore_order: ")
+        self.logger.info("------ expected_json: ")
+        self.logger.info(self.pformat(expected_json))
+        self.logger.info("------ actual_json: ")
+        self.logger.info(self.pformat(actual_json))
+
+        # from collections import OrderedDict
+        # def sorting(item):
+        #     if isinstance(item, dict):
+        #         return sorted((key, sorting(values)) for key, values in item.items())
+        #     if isinstance(item, list):
+        #         return sorted(sorting(x) for x in item)
+        #     else:
+        #         return item
+        #
+
+        diff = self.compare_json_and_return_diff(json.loads(json.dumps(expected_json, sort_keys=True)),
+                                                 json.loads(json.dumps(actual_json, sort_keys=True)))
+        assert diff == {}
